@@ -19,6 +19,7 @@
 import signal
 
 from functools import wraps
+from types import MethodType
 from threading import Condition, Lock
 
 
@@ -101,6 +102,124 @@ def sighandler(signals):
     return wrap
 
 
+# --------------------------------------------------------------------------- #
+#                               Common Functions                              #
+# --------------------------------------------------------------------------- #
+def new(self, *args):
+    self._old(*args)
+    with self._external_lock:
+        self._external_lock.notify_all()
+
+
+def waitfortasks(tasks, timeout=None):
+    """Waits for one or more *Task* to be ready or until *timeout* expires.
+
+    *tasks* is a list containing one or more *pebble.Task* objects.
+    If *timeout* is not None the function will block
+    for the specified amount of seconds.
+
+    The function returns a list containing the ready *Tasks*.
+
+    """
+    block = Condition(Lock())
+    ready = lambda: [t for t in tasks if t.ready]
+
+    for task in tasks:
+        task._external_lock = block
+        with task._task_ready:
+            task._old = task._set
+            task._set = MethodType(new, task)
+
+    with block:
+        if len(ready()) == 0:
+            block.wait(timeout)
+
+    for task in tasks:
+        with task._task_ready:
+            task._set = task._old
+            delattr(task, '_old')
+            delattr(task, '_external_lock')
+
+    return ready()
+
+
+def waitforthreads(threads, timeout=None):
+    """Waits for one or more *Thread* to exit or until *timeout* expires.
+
+    .. note::
+
+       Expired *Threads* are not joined by *waitforthreads*.
+
+    *threads* is a list containing one or more *threading.Thread* objects.
+    If *timeout* is not None the function will block
+    for the specified amount of seconds.
+
+    The function returns a list containing the ready *Threads*.
+
+    """
+    block = Condition(Lock())
+    ready = lambda: [t for t in threads if not t.is_alive()]
+
+    for thread in threads:
+        thread._external_lock = block
+        if hasattr(thread, '_stop'):
+            with thread._block:
+                thread._old = thread._stop
+                thread._stop = MethodType(new, thread)
+        else:
+            with thread._Thread__block:
+                thread._old = thread._Thread__stop
+                thread._Thread__stop = MethodType(new, thread)
+
+    with block:
+        if len(ready()) == 0:
+            block.wait(timeout)
+
+    for thread in threads:
+        if hasattr(thread, '_stop'):
+            with thread._block:
+                thread._stop = thread._old
+        else:
+            with thread._Thread__block:
+                thread._Thread__stop = thread._old
+        delattr(thread, '_old')
+        delattr(thread, '_external_lock')
+
+    return ready()
+
+
+def waitforqueues(queues, timeout=None):
+    """Waits for one or more *Queue* to be ready or until *timeout* expires.
+
+    *queues* is a list containing one or more *Queue.Queue* objects.
+    If *timeout* is not None the function will block
+    for the specified amount of seconds.
+
+    The function returns a list containing the ready *Queues*.
+
+    """
+    block = Condition(Lock())
+    ready = lambda: [q for q in queues if not q.empty()]
+
+    for queue in queues:
+        queue._external_lock = block
+        with queue.mutex:
+            queue._old = queue.put
+            queue.put = MethodType(new, queue)
+
+    with block:
+        if len(ready()) == 0:
+            block.wait(timeout)
+
+    for queue in queues:
+        with queue.mutex:
+            queue.put = queue._old
+        delattr(queue, '_old')
+        delattr(queue, '_external_lock')
+
+    return ready()
+
+
 # --------------------------------------------------------------------------- #
 #                               Common Objects                                #
 # --------------------------------------------------------------------------- #
