@@ -3,8 +3,13 @@ import time
 import signal
 import unittest
 import threading
+try:  # Python 2
+    from Queue import Queue
+except:  # Python 3
+    from queue import Queue
 
-from pebble import synchronized, sighandler
+from pebble import synchronized, sighandler, thread
+from pebble import waitfortasks, waitforthreads, waitforqueues
 from pebble import Task, TimeoutError, TaskCancelled
 
 
@@ -34,6 +39,25 @@ except ImportError:
     pass
 
 
+@thread.spawn
+def thread_function(value):
+    time.sleep(value)
+    return value
+
+
+@thread.spawn
+def queue_function(queues, index, value):
+    time.sleep(value)
+    queues[index].put(value)
+    return value
+
+
+@thread.concurrent
+def concurrent_function(value):
+    time.sleep(value)
+    return value
+
+
 class TestSynchronizedDecorator(unittest.TestCase):
     def test_wrapper_decorator_docstring(self):
         """Synchronized docstring of the original function is preserved."""
@@ -80,6 +104,97 @@ class TestSigHandler(unittest.TestCase):
             self.assertEqual(results, 1)
 
 
+class TestWaitForTasks(unittest.TestCase):
+    def setUp(self):
+        pass
+
+    def test_waitfortasks_single(self):
+        """Waitfortasks waits for a single task."""
+        task = concurrent_function(0.01)
+        self.assertEqual(waitfortasks([task])[0], task)
+
+    def test_waitfortasks_multiple(self):
+        """Waitfortasks waits for multiple tasks."""
+        tasks = []
+        for _ in range(5):
+            tasks.append(concurrent_function(0.01))
+        time.sleep(0.1)
+        self.assertEqual(waitfortasks(tasks), tasks)
+
+    def test_waitfortasks_timeout(self):
+        """Waitfortasks returns empty list if timeout."""
+        task = concurrent_function(0.1)
+        self.assertEqual(waitfortasks([task], timeout=0.01), [])
+
+    def test_waitfortasks_restore(self):
+        """Waitfortasks Task object is restored to original one."""
+        task = concurrent_function(0.01)
+        expected = sorted(dir(task))
+        waitfortasks([task])
+        self.assertEqual(sorted(dir(task)), expected)
+
+
+class TestWaitForThreads(unittest.TestCase):
+    def setUp(self):
+        pass
+
+    def test_waitforthreads_single(self):
+        """Waitforthreads waits for a single thread."""
+        thread = thread_function(0.01)
+        self.assertEqual(waitforthreads([thread])[0], thread)
+
+    def test_waitforthreads_multiple(self):
+        """Waitforthreads waits for multiple threads."""
+        threads = []
+        for _ in range(5):
+            threads.append(thread_function(0.01))
+        time.sleep(0.1)
+        self.assertEqual(waitforthreads(threads), threads)
+
+    def test_waitforthreads_timeout(self):
+        """Waitforthreads returns empty list if timeout."""
+        thread = thread_function(0.1)
+        self.assertEqual(waitforthreads([thread], timeout=0.01), [])
+
+    def test_waitforthreads_restore(self):
+        """Waitforthreads Thread object is restored to original one."""
+        thread = thread_function(0)
+        time.sleep(0.01)
+        expected = sorted(dir(thread))
+        waitforthreads([thread])
+        self.assertEqual(sorted(dir(thread)), expected)
+
+
+class TestWaitForQueues(unittest.TestCase):
+    def setUp(self):
+        self.queues = (Queue(), Queue(), Queue())
+
+    def test_waitforqueues_single(self):
+        """Waitforqueues waits for a single queue."""
+        queue_function(self.queues, 0, 0.01)
+        self.assertEqual(waitforqueues(self.queues)[0], self.queues[0])
+
+    def test_waitforqueues_multiple(self):
+        """Waitforqueues waits for multiple queues."""
+        for index in range(3):
+            queue_function(self.queues, index, 0.01)
+        time.sleep(0.1)
+        self.assertEqual(waitforqueues(self.queues), list(self.queues))
+
+    def test_waitforqueues_timeout(self):
+        """Waitforqueues returns empty list if timeout."""
+        queue_function(self.queues, 0, 0.01)
+        self.assertEqual(waitforqueues(self.queues, timeout=0.01), [])
+
+    def test_waitforqueues_restore(self):
+        """Waitforqueues Queue object is restored to original one."""
+        queue_function(self.queues, 0, 0.01)
+        time.sleep(0.01)
+        expected = sorted(dir(self.queues[0]))
+        waitforqueues(self.queues)
+        self.assertEqual(sorted(dir(self.queues[0])), expected)
+
+
 class TestTask(unittest.TestCase):
     def setUp(self):
         self.task = Task(0, None, None, None, None, None, None)
